#define __CS 10
#define __DC 9


#include <EdWgfx.h>


// Color definitions
#define  BLACK   0
#define BLUE    2
#define GREEN   4
#define CYAN    9
#define MAGENTA 10
#define RED     11
#define WHITE   14
#define YELLOW  15


EdWgfx tft = EdWgfx(__CS, __DC);


byte errorCode;
void setup() {
  Serial.begin(38400);
  long unsigned debug_start = millis ();
  while (!Serial && ((millis () - debug_start) <= 5000)) ;
  tft.begin();
}


void loop(void) {
  uint8_t rd[38];
  tft.readId( rd );
  byte rd0, rd1, rd2, rd3, pusty;
  byte id[6], ver[4], typ[2];
  word cfg, dispX, dispY;
  word sx, sy, ex, ey;
  word cursX, cursY, colorPen, colorBack;
  byte io, pwmIn;
  word adrDma;
  

  uint8_t p = 0;
  adrDma = rd[p++] << 8;	// #0
  adrDma |= rd[p++];
  id[0] = rd[p++];  // #2
  id[1] = rd[p++];
  id[2] = rd[p++]; // #4
  id[3] = rd[p++];
  id[4] = rd[p++];
  id[5] = rd[p++];

  ver[0] = rd[p++];	// #8
  ver[1] = rd[p++];
  ver[2] = rd[p++];
  ver[3] = rd[p++];

  typ[1] = rd[p++];	// #12
  typ[0] = rd[p++];
  cfg = rd[p++] << 8;
  cfg |= rd[p++];
  
  dispX = rd[p++] << 8;	// #16
  dispX |= rd[p++];
  dispY = rd[p++] << 8;
  dispY |= rd[p++];
  
  sx = rd[p++];		// #20
  sx |= rd[p++];
  sy = rd[p++] << 8;
  sy |= rd[p++];
  
  ex = rd[p++] << 8;	// #24
  ex |= rd[p++];
  ey = rd[p++] << 8;	
  ey |= rd[p++];
  
  cursX = rd[p++] << 8;	// #28
  cursX |= rd[p++];
  cursY = rd[p++] << 8;
  cursY |= rd[p++];
  
  colorPen = rd[p++] << 8;	// #32
  colorPen |= rd[p++];
  colorBack = rd[p++] << 8;
  colorBack |= rd[p++];
  
  pwmIn = rd[p++];		// #36
  io = rd[p++];

  char txt[100];
  Serial.println("Read:");
  for (uint8_t x = 0; x < 40; x++) {
    sprintf( txt, "%02x ", rd[x] ); if ( x % 16 || !x ) Serial.print(txt); else Serial.println(txt);
  }

  if ( ! id[3] ) id[3] = ' ';
  sprintf( txt, "\n\r\n\rID='%c%c%c%c' Ver='%c%c%c' Typ=%x%02x", id[0], id[1], id[2], id[3], ver[0], ver[1], ver[2], typ[1], typ[0] ); Serial.println(txt);
  sprintf( txt, "CFG=%x Szer=%d Wys=%d", cfg, dispX, dispY ); Serial.println(txt);
  sprintf( txt, "Okno=%x,%d do %d,%d Cursor=%d,%d, kolory=%d,%d", sx, sy, ex, ey, cursX, cursY, colorPen, colorBack ); Serial.print(txt);
  sprintf( txt, " PWM=%d IO=%x \n\r", pwmIn, io ); Serial.println(txt);


  //errorCode = tft.errorCode();
  if (errorCode != 0) {
    Serial.print("Init error! ");
    if (bitRead(errorCode, 0)) Serial.print("MOSI or SCLK pin mismach!\n");
    if (bitRead(errorCode, 1)) Serial.print("CS or DC pin mismach!\n");
  } else {
    Serial.println(F("Benchmark                Time (microseconds)"));
  }
  if (errorCode == 0) {
    Serial.print(F("Screen fill              "));
    Serial.println(testFillScreen());
    //delay(1000);

    Serial.print(F("Text                     "));
    Serial.println(testText());
    delay(10000);

    Serial.print(F("Text2                    "));
    Serial.println(testText2());
    //delay(3000);

    Serial.print(F("Lines                    "));
    Serial.println(testLines(CYAN));
    //delay(500);

    Serial.print(F("Horiz/Vert Lines         "));
    Serial.println(testFastLines(RED, BLUE));
    //delay(500);

    Serial.print(F("Rectangles (outline)     "));
    Serial.println(testRects(GREEN));
    //delay(500);

    Serial.print(F("Rectangles (filled)      "));
    Serial.println(testFilledRects(YELLOW, MAGENTA));
    //delay(500);

    Serial.print(F("Circles (filled)         "));
    Serial.println(testFilledCircles(10, MAGENTA));

    Serial.print(F("Circles (outline)        "));
    Serial.println(testCircles(10, WHITE));
    //delay(500);

    Serial.print(F("Triangles (outline)      "));
    Serial.println(testTriangles());
    //delay(500);

    Serial.print(F("Triangles (filled)       "));
    Serial.println(testFilledTriangles());
    //delay(500);

    Serial.print(F("Rounded rects (outline)  "));
    Serial.println(testRoundRects());
    //delay(500);

    Serial.print(F("Rounded rects (filled)   "));
    Serial.println(testFilledRoundRects());
    //delay(500);

    Serial.println(F("Done!"));
  }

  /*
    for (uint8_t rotation = 0; rotation < 4; rotation++) {
      tft.setRotation(rotation);
      testText();
      //delay(2000);
    }
  */
  delay(1000);
}


unsigned long testFillScreen() {
  unsigned long start = micros();
  tft.fillScreen(0);
  tft.display(); delay(35);

  tft.fillScreen(RED);
  tft.display(); delay(35);

  tft.fillScreen(GREEN);
  tft.display(); delay(35);

  tft.fillScreen(BLUE);
  tft.display(); delay(35);

  unsigned long t = micros() - start;


  for (byte x = 0; x < 16; x++) {
    tft.fillScreen(x);
    tft.display(); delay(50);
  }


  return t;
}


unsigned long testText() {
  tft.fillScreen(0);
  unsigned long start = micros();

  tft.setTextAtrybut( TFT_MODE_PROP ); // | TFT_MODE_TXT_BACK
  tft.setCursor(0, 0);
  tft.setTextColor(BLUE);
  tft.setTextSize(1);
  tft.printString("Hello World!");

  tft.setTextColor(YELLOW);
  tft.setCursor(0, 16);
  tft.setTextAtrybut( 0 );
  tft.printString("Hello World!");

  tft.setTextAtrybut( TFT_MODE_PROP ); // | TFT_MODE_TXT_BACK
  tft.setCursor(0, 32);
  tft.setTextColor(GREEN);
  tft.setTextSize(2);
  tft.printString("Hello");
  tft.setTextSize(1);
  tft.setCursor(75, 32);
  tft.setTextColor(YELLOW);
  tft.printString("Hello");
  tft.setCursor(75, 32 + 16);
  tft.setTextColor(BLUE);
  tft.printString("Hello");

  tft.setCursor(0, 64);
  tft.setTextColor(RED);
  tft.setTextSize(0);
  char txt[10];
  tft.setTextAtrybut( TFT_MODE_PROP | TFT_MODE_TXT_BACK );
  tft.setTextBack(WHITE);
  sprintf( txt, "Czcionka 5x7 z t\x04""em");
  tft.printString(txt);

  tft.setCursor(0, 75);
  tft.setTextColor(WHITE);
  tft.setTextSize(3);
  tft.setTextBack(BLUE);
  sprintf( txt, "Czcionka 10x7 z t\x04""em");
  tft.printString(txt);

  tft.setTextColor(YELLOW);
  tft.setTextSize(3);
  tft.setTextAtrybut( TFT_MODE_PROP );
  tft.setCursor(0, 90);
  sprintf( txt, "\x04""\x06""d\x08"" po polsku");
  tft.printString(txt);


  tft.display(); delay(35);

  return micros() - start;
}


unsigned long testText2() {
  tft.fillScreen(0);
  unsigned long start = micros();
  /*
    tft.setCursor(0, 0);
    tft.setTextColor(WHITE);
    tft.setTextSize(2);
    tft.println("I implore thee,");
    tft.setTextSize(1);
    tft.println("my foonting turlingdromes.");
    tft.println("And hooptiously drangle me");
    tft.println("with crinkly bindlewurdles,");
    tft.println("Or I will rend thee");
    tft.println("in the gobberwarts");
    tft.println("with my blurglecruncheon,");
    tft.println("see if I don't!");

    tft.display(); delay(35);
  */
  return micros() - start;
}


unsigned long testLines(uint16_t color) {
  unsigned long start, t;
  int           x1, y1, x2, y2,
                w = tft.width(),
                h = tft.height();

  tft.fillScreen(0);

  x1 = y1 = 0;
  y2    = h - 1;
  start = micros();
  for (x2 = 0; x2 < w; x2 += 6) tft.drawLine(x1, y1, x2, y2, color);
  x2    = w - 1;
  for (y2 = 0; y2 < h; y2 += 6) tft.drawLine(x1, y1, x2, y2, color);
  t     = micros() - start; // fillScreen doesn't count against timing

  tft.display(); delay(35);

  tft.fillScreen(0);

  x1    = w - 1;
  y1    = 0;
  y2    = h - 1;
  start = micros();
  for (x2 = 0; x2 < w; x2 += 6) tft.drawLine(x1, y1, x2, y2, color);
  x2    = 0;
  for (y2 = 0; y2 < h; y2 += 6) tft.drawLine(x1, y1, x2, y2, color);
  t    += micros() - start;

  tft.display(); delay(35);

  tft.fillScreen(0);

  x1    = 0;
  y1    = h - 1;
  y2    = 0;
  start = micros();
  for (x2 = 0; x2 < w; x2 += 6) tft.drawLine(x1, y1, x2, y2, color);
  x2    = w - 1;
  for (y2 = 0; y2 < h; y2 += 6) tft.drawLine(x1, y1, x2, y2, color);
  t    += micros() - start;

  tft.display(); delay(35);

  tft.fillScreen(0);

  x1    = w - 1;
  y1    = h - 1;
  y2    = 0;
  start = micros();
  for (x2 = 0; x2 < w; x2 += 6) tft.drawLine(x1, y1, x2, y2, color);
  x2    = 0;
  for (y2 = 0; y2 < h; y2 += 6) tft.drawLine(x1, y1, x2, y2, color);

  tft.display(); delay(35);

  return micros() - start;
}


unsigned long testFastLines(uint16_t color1, uint16_t color2) {
  unsigned long start;
  int           x, y, w = tft.width(), h = tft.height();

  tft.fillScreen(0);
  start = micros();
  for (y = 0; y < h; y += 5) tft.drawFastHLine(0, y, w, color1);
  for (x = 0; x < w; x += 5) tft.drawFastVLine(x, 0, h, color2);

  tft.display(); delay(35);

  return micros() - start;
}


unsigned long testRects(uint16_t color) {
  unsigned long start;
  int           n, i, i2,
                cx = tft.width()  / 2,
                cy = tft.height() / 2;

  tft.fillScreen(0);
  n     = min(tft.width(), tft.height());
  start = micros();
  for (i = 2; i < n; i += 6) {
    i2 = i / 2;
    tft.drawRect(cx - i2, cy - i2, i, i, color);
  }

  tft.display(); delay(35);

  return micros() - start;
}


unsigned long testFilledRects(uint16_t color1, uint16_t color2) {
  unsigned long start, t = 0;
  int           n, i, i2,
                cx = (tft.width()  / 2) - 1,
                cy = (tft.height() / 2) - 1;

  tft.fillScreen(0);
  n = min(tft.width(), tft.height());
  for (i = n; i > 0; i -= 6) {
    i2    = i / 2;
    start = micros();
    tft.fillRect(cx - i2, cy - i2, i, i, color1);
    t    += micros() - start;
    // Outlines are not included in timing results
    tft.drawRect(cx - i2, cy - i2, i, i, color2);
  }
  tft.display(); delay(35);

  return t;
}


unsigned long testFilledCircles(uint8_t radius, uint16_t color) {
  unsigned long start;
  int x, y, w = tft.width(), h = tft.height(), r2 = radius * 2;

  tft.fillScreen(0);
  start = micros();
  /*
    for (x = radius; x < w; x += r2) {
    for (y = radius; y < h; y += r2) {
      tft.fillCircle(x, y, radius, color);
    }
    }

    tft.display(); delay(35);
  **/
  return micros() - start;
}


unsigned long testCircles(uint8_t radius, uint16_t color) {
  unsigned long start;
  int           x, y, r2 = radius * 2,
                      w = tft.width()  + radius,
                      h = tft.height() + radius;

  // Screen is not cleared for this one -- this is
  // intentional and does not affect the reported time.
  start = micros();
  /*
    for (x = 0; x < w; x += r2) {
    for (y = 0; y < h; y += r2) {
      tft.drawCircle(x, y, radius, color);
    }
    }

    tft.display(); delay(35);
  */
  return micros() - start;
}

unsigned long testTriangles() {
  unsigned long start;
  int           n, i, cx = tft.width()  / 2 - 1,
                      cy = (tft.height() / 2) - 1;

  tft.fillScreen(0);
  n     = min(cx, cy);
  start = micros();
  /*
    for (i = 0; i < n; i += 5) {
    tft.drawTriangle(
      cx    , cy - i, // peak
      cx - i, cy + i, // bottom left
      cx + i, cy + i, // bottom right
      tft.Color565(0, 0, i));
    }

    tft.display(); delay(35);
  */
  return micros() - start;
}

unsigned long testFilledTriangles() {
  unsigned long start, t = 0;
  int           i, cx = (tft.width() / 2) - 1,
                   cy = tft.height() / 2 - 1;


  tft.fillScreen(0);
  start = micros();
  /*
    for (i = min(cx, cy); i > 10; i -= 5) {
    start = micros();
    tft.fillTriangle(cx, cy - i, cx - i, cy + i, cx + i, cy + i, i);
    t += micros() - start;
    tft.drawTriangle(cx, cy - i, cx - i, cy + i, cx + i, cy + i, i);
    }

    tft.display(); delay(35);
  */
  return t;
}

unsigned long testRoundRects() {
  unsigned long start;
  int           w, i, i2,
                cx = (tft.width()  / 2) - 1,
                cy = (tft.height() / 2) - 1;

  tft.fillScreen(0);
  w     = min(tft.width(), tft.height());
  start = micros();
  /*
    for (i = 0; i < w; i += 6) {
    i2 = i / 2;
    tft.drawRoundRect(cx - i2, cy - i2, i, i, i / 8, i );
    tft.display(); delay(35);

    }

    tft.display(); delay(35);
  */
  return micros() - start;
}


unsigned long testFilledRoundRects() {
  unsigned long start;
  int           i, i2,
                cx = (tft.width()  / 2) - 1,
                cy = (tft.height() / 2) - 1;

  tft.fillScreen(0);
  start = micros();
  /*
    for (i = min(tft.width(), tft.height()); i > 20; i -= 6) {
    i2 = i / 2;
    tft.fillRoundRect(cx - i2, cy - i2, i, i, i / 8, tft.Color332(0, i, 0));
    }

    tft.display(); delay(35);
  */
  return micros() - start;
}
